###################################################################
# $Id$
# $Log$
###################################################################
package GotoForm;

require 5.001;

###################################################################
sub new
{
    local($class, $cgi)=@_;
    local $self={};
    $cgi="goto.cgi" if($cgi eq '');
    $self->{'cgi'}=$cgi;
    bless $self, $class;
}

###################################################################
sub ReadConf
{
    local($self, $infile)=@_;
    local(%files, $file, $texte);

    $infile=$self->{'conf'} if (!$infile);
    $infile='page_list' if (!$infile);
    open(INFILE, "<$infile") or die ("Unable to open $infile, $!");
    $self->{'conf'}=$infile;
    while(<INFILE>)
    {
        chomp;
        s/^[ \t]+//;                    # drop leading space
        next if(/^#/);                  # ignore comments
        ($file, $texte)=split(/ /, $_, 2);
        $files{$file}=[$texte, $.];     # keep linenum, for sorting
    }
    close(INFILE);
    $self->{'files'}={%files};
    return %files;
}

###################################################################
sub DropForm
{
    local($self)=@_;
    local($out);
    $out=$self->tempForm;
    $out=~s/<!-- top -->/<SELECT name=page size=1>/;
    $out=~s/<!-- option -->/<OPTION>/gi;
    $out=~s/<!-- disabled -->/<OPTION disabled>/gi;
    $out=~s/<!-- bottom -->/<\/SELECT>/;
    return $out;
}

###################################################################
sub SelectForm
{
    local($self, $size)=@_;
    local($out);

    $size=$#{keys %{$self->{files}}} if(!$size);

    $out=$self->DropForm;
    $out=~s/size=1>/size=$size>/;
    return $out;
}

###################################################################
sub RadioForm
{
    local($self)=@_;
    local($out);
    $out=$self->tempForm;
    $out=~s/<!-- top -->//;
    $out=~s/<!-- option -->(.*)\n/<br><INPUT type=radio name=page value="\1">\1\n/gi;
    $out=~s/<!-- disabled -->/<br>/gi;
    $out=~s/<!-- bottom -->//;
    return $out;
}

###################################################################
sub tempForm
{
    local($self)=@_;
    local(%files, $page, $out, $cgi);
    local($package, $filename, $line)=caller 1;
    %files=%{$self->{'files'}};

    $cgi=$self->{'cgi'};

    $out="<FORM method=post action=\"$cgi\">\n";
    if(!$self->{'conf'})
    {
        warn "Configuration file is not defined at $filename line $line.\n";
    } else
    {
        $out.='<INPUT TYPE=hidden NAME=config VALUE="'.$self->{'conf'}."\">\n";
    }
    $out.="<!-- top -->";
    foreach $page (sort {$files{$a}[1] <=> $files{$b}[1];} keys %files)
    {
        if($page eq 'disabled')
        {
            $out.="\n<!-- disabled -->";
        } else
        {
            $out.="\n<!-- option -->";
        }
        $out.=$files{$page}[0];
    }

    $out.="\n<!-- bottom --><BR><INPUT type=submit value=Oui></FORM>\n";
    return $out;
}

###################################################################
sub page2name
{
    local($self, $page)=@_;
    return $self->{'files'}->{$page};
}

###################################################################
sub name2page
{
    local($self, $name)=@_;
    local($page, $n);

    $name=~s/([\x80-\xff])/'&#'.ord($1).';'/ge;

    foreach $page (keys %{$self->{'files'}})
    {
        $n=$self->{'files'}->{$page}->[0];

        if($n eq $name)
        {
            return $page;
        }
    }
    return undef;
}
1;

__END__

=head1 NAME

GotoForm.pm - A simple navigation tool

=head1 DESCRITION

GotoForm provides a simple logic to implement a navigational tool for a
www site.  An ascii file contains information regarding pages and there
titles.  This module is used to present these titles in various
HTML forms (drop down list, radio buttons).  It is also used to look-up the
information returned by the form to get the associated URL.

The file format is:

    # comments begin with a pound
    # URL Title
    page1.html Titles come after the first whitespace, and may contain white space
    disable A disabled URL
    disable ------------------------------
    ../en/index.html A different page
    http://www.cam.org A full URL

=head1 METHODS

L<new> --
L<ReadConf> --
L<DropForm> --
L<SelectForm> --
L<RadioForm> --
L<tempForm> --
L<page2name> --
L<name2page>

=item new ([$cginame])

Creates a new object.  $cginame is the URL that should be used when
creating an HTML form.  Default value is "goto.cgi".  It goes as the
value of the ACTION attribut of the FORM tag.  ie:
<FORM method=post action=$cginame>

=item ReadConf ([$config_file])

Reads the URL/Title information from $config_file.  If $config_file,
isn't specified, 'page_list' is used.

=item DropForm

Generates an HTML form using SELECT/OPTION tags.

sub SelectForm ($size)

Same as DropForm, but creates a multiline selection box instead.

sub RadioForm

Creates a list of radio buttons from the current information.

sub page2name

Given the URL of a page, returns it's title.

sub name2page

Given the name of a page, returns the URL.  A CGI form will return the
name of the page, so this will provide the lookup.

=head1 INSTALLATION

=cut
